function [RMatrix1, RMatrix2, zMatrix1,zMatrix2, matrix12_diff_z, matrix12_diff_P] = ...
    gretna_matrix2(group1, group2, num_rand);

% *************************************************************************
% The function is used to calculate the differences in the correlation
% matrix between the groups. Each element of the correlation matrix
% represents the correlation coefficient in cortical thickness between two
% brain region across the subjects.
%
% function [RMatrix1, RMatrix2, zMatrix1,zMatrix2, matrix12_diff_z,
% matrix12_diff_P] = gretna_matrix2 (group1, group2, num_rand); 
% 
% Input:
%           group1:
%                       s*N matrix of normal controls (s: the number of
%                       subjects in group1; N: the number of brain regions)
%           group2:
%                       s*N matrix of patients  (s: the number of subjects
%                       in group2; N: the number of brain regions)
%           num_random:
%                       The number of random experiments (default: 1000);
% output:
%           RMatrix1:
%                       Correlation matrix in group1 (N*N);
%           zMatrix1:
%                       The fisher's transformation of RMatrix1(N*N);
%           RMatrix2:
%                       Correlation matrix in group2(N*N);
%           zMatrix2:
%                       The fisher's transformation of RMatrix2(N*N);
%           matrix12_diff_z:
%                       The inter-regional correlation difference (z
%                      score, uncorrected) between group1 and group2 (N*N)
%           matrix12_diff_P:
%                       The statistical difference (p value) in the
%                       correlation between the two groups.
%                       matrix12_diff_P.OneTailed_P: one-tailed(N*N);
%                       matrix12_diff_P.TwoTailed_P: two-tailed(N*N);
% 
% Example: [RMatrix1, RMatrix2, zMatrix1,zMatrix2, matrix12_diff_z,
% matrix12_diff_P] = gretna_matrix2 (group1, group2);  
% 
% Yong HE, BIC,MNI, McGill 2006/09/12
% *************************************************************************


%warning off MATLAB:divideByZero
warning off

if nargin < 2
    error('No input parameters');end

if nargin == 2
    num_rand = 10000;
end

if nargin > 3
    error('Error the number of input parameters (< 3).');
end

NR1 = size(group1,2);
NR2 = size(group2,2);
if NR1~=NR2
    error('Error the number of regions between the two groups');
end

N1 = size(group1,1);
N2 = size(group2,1);

N_tgroup = N1 + N2;
tgroup_Reg = [group1;group2];

r1 = corrcoef(group1);
r2 = corrcoef(group2);

% Firsher's r to z transformation
wp = log(((1+r1).*(1-r2))./((1-r1).*(1+r2)))/(2*sqrt(1/(N1-3)+1/(N2-3)));
zMatrix1 = 0.5*log((1+r1)./(1-r1));
zMatrix2 = 0.5*log((1+r2)./(1-r2));

% random permutation test
wp_rand = zeros(NR1,NR1,num_rand);
for i = 1: num_rand
    i
    rp = randperm(N_tgroup);
    rerand_1_Reg = tgroup_Reg(rp(1:N1),:);
    rerand_2_Reg = tgroup_Reg(rp ((N1+1):N_tgroup),:);
    rerand_corr_1 = corrcoef(rerand_1_Reg);
    rerand_corr_2 = corrcoef(rerand_2_Reg);
    wp_rand(:,:,i) = log(((1+rerand_corr_1).*(1-rerand_corr_2))./...
        ((1-rerand_corr_1).*(1+rerand_corr_2)))/(2*sqrt(1/(N1-3)+1/(N2-3)));
end

wpmean = mean(wp_rand,3);
wpstd = std(wp_rand,0,3);

matrix12_diff_z = (wp-wpmean)./wpstd;
matrix12_diff_z(find(isnan(matrix12_diff_z)==1))=0;
matrix12_diff_P.OneTailed_P = 1 - spm_Ncdf(abs(matrix12_diff_z));
matrix12_diff_P.TwoTailed_P = 2*matrix12_diff_P.OneTailed_P;

RMatrix1 = r1;
RMatrix2 = r2;

return

