function [M] = gretna_readthickness1(sourcedir, IDfile, thicknessdir, prefix, isnative, fwhm)

% *************************************************************************
% This function is used to read individual cortical thickness values
% typically obtained from the CIVET procedure.
% 
% function [M] = gretna_readthickness(sourcedir, IDfile, thicknessdir,
% prefix, isnative, FWHM)
%     
% For example: sourcedir =
% '/data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/'; IDfile: 'IDfile'.
% Here "IDfile" is the ID number of each subject. It
%           should be is a text file including the ID of each subject. See
%           an example file (subID) of this directory.
% thicknessdir = '/surface_stats/'; prefix = 'mci'; isnative = 1; fwhm =
% 30; [M] = gretna_readthickness(sourcedir, subID, thicknessdir, prefix,
% isnative, fwhm);
% 
% If you choose the above input parameters, you will read the cortical
% thickness values from the following the files:
% 
% /data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/01/surface_stats/
% mci_01_rms_thickness_native_81920_30mmFWHM.txt
% 
% /data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/02/surface_stats/
% mci_02_rms_thickness_native_81920_30mmFWHM.txt ......
% 
% input: 
%           sourcedir:  the directory of each subject. e.g.
%                           /data/aces/aces1/yonghe/eld_mci/pipe1004/output
%                           mci/; 
%           subID:      a txt file including the ID of each subject. It
%                          should be the name of subdirectory under the
%                          input parameter "sourcedir". If you open this
%                          txt file. It should be like this: 01 02 03 04...
%                          or like this:
%                           01 02 03 04 ...
%           thicknessdir: the directory of cortical thickness files
%                          for each subject. It should be "/surface_stats/"
%                          or "/surface_registration/"
%           prefix:      the prefix name of cortical thickness file. e.g.
%                          in the filename
%                          "mci_01_rms_thickness_native_81920_30mmFWHM.txt"
%                          ,  prefix = 'mci';
%           isnative:   whether cortical thickness value was measured in
%                           native space. If so, isnative = 1 (default);
%                           otherwise isnative = 0;
%                          
%           fwhm:       the diffusion smooth kernal. 
%                           0: no smooth; 10: 10 mm diffusion smooth kernal
%                           20: 20 mm diffusion smooth kernal 30: 30 mm
%                           diffusion smooth kernal 40: 40 mm diffusion
%                           smooth kernal
% 
% output:
%           M:           an s*n matrix with cortical thickness values for
%                          each node for each subject. n: the number of
%                          nodes (e.g. 81920); s: the number of subjects.
%                     
% Yong HE, BIC, MNI, McGill 2006/09/12
% 

 
if nargin < 4
    error('No parameters');end

if nargin == 4 
    isnative = 1; fwhm = 30; end

if nargin == 5  
    fwhm = 30; end

if nargin > 6 
    error('Error the number of input parameters (< 6).'); 
end

if ~strcmp(thicknessdir, '\surface_stats\') & ...
        ~strcmp(thicknessdir, '\surface_registration\')
    error('Error "thicknessdir" parameter');
end

if isnative == 1
    native = 'native';
elseif isnative ==0
    native = [];    
else
    error('Error "isnative" parameter (1 or 0)');
end

switch fwhm
    case {10,20,30,40}
        fwhm = int2str(fwhm);
    case {0}
        fwhm = [];
    otherwise
        error('Unknown fwhm smooth kerna (0,10,20,30 or 40)l!');
end

subID = textread(IDfile, '%s');
Nsub = size(subID,1);

if length(fwhm) == 0 && isnative == 0
    for i = 1:Nsub
        filename1 = strcat(strcat(strcat(strcat(prefix,'_'),subID{i}),'_rms_thickness'),native);
        filename2 = strcat(filename1,'_81920.txt');
        filename3 = strcat(strcat(sourcedir,subID{i}),thicknessdir);
        filename = strcat(filename3, filename2);
        M(:,i) = load(filename);
    end
elseif length(fwhm) == 0 && isnative == 1
    for i = 1:Nsub
        filename1 = strcat(strcat(strcat(strcat(prefix,'_'),subID{i}),'_rms_thickness_'),native);
        filename2 = strcat(filename1,'_81920.txt');
        filename3 = strcat(strcat(sourcedir,subID{i}),thicknessdir);
        filename = strcat(filename3, filename2);
        M(:,i) = load(filename);
    end
else
    for i = 1:Nsub
        filename1 = strcat(strcat(strcat(strcat(prefix,'_'),subID{i}),'_rms_thickness_'),native);
        filename2 = strcat(strcat('_81920_',fwhm),'mmFWHM.txt');
        filename3 = strcat(strcat(sourcedir,subID{i}),thicknessdir);
        filename = strcat(filename3,strcat(filename1,filename2));
        M(:,i) = load(filename);
    end
end
    M = M';







