function [M_left, M_right] = gretna_readthickness2_jc(sourcedir, IDfile, thicknessdir, prefix, isnative, fwhm, rsl)

% *************************************************************************
% This function is used to read individual cortical thickness values
% typically obtained from the CIVET pipeline procedure (Claude's version).
%
% function [M] = gretna_readthickness2(sourcedir, IDfile, thicknessdir,
% prefix, isnative, FWHM, rsl)
%
% input:
%           sourcedir:
%                           the directory of each subject. e.g.
%                           /data/aces/aces1/yonghe/eld_mci/pipe1004/output
%                           mci/;
%           subjects:
%                           a txt file including the ID of each subject. It
%                           should be the name of subdirectory under the
%                           input parameter "sourcedir". If you open this
%                           txt file. It should be like this: 01 02 03
%                           04... or like this:  01 02 03 04 ...
%           thicknessdir:
%                           the directory saving cortical thickness values
%                           for each subject. It should be
%                           "/surface_stats/" or "/thickness/" 
%           prefix:       
%                           the prefix name of cortical thickness file.
%                           e.g. in the filename
%                           "mci_01_native_rms_tlink_30mm_left.txt", prefix
%                           = 'mci';
%           isnative:   
%                           whether cortical thickness value was measured
%                           in native space. If so, isnative = 1 (default);
%                           otherwise isnative = 0; 
%           fwhm:       
%                           the diffusion smooth kernal. 0.0: no smooth; 
%                           10: 10 mm diffusion smooth kernal; 20: 20 mm
%                           diffusion smooth kernal; 30: 30 mm diffusion
%                           smooth kernal; 40: 40 mm diffusion smooth
%                           kernal;
%           rsl:            surface registration file. [rsl =[],default].
%
% output:
%           M_left:     
%                           an s*n matrix with cortical thickness values
%                           for each node for each subject (left
%                           hemisphere). n: the number of nodes (e.g.
%                           81920); s: the number of subjects.
%           M_right:   
%                           an s*n matrix with cortical thickness values
%                           for each node for each subject (right
%                           hemisphere). n: the number of nodes (e.g.
%                           81920); s: the number of subjects. 
%
% For example:
% 
%  sourcedir =
%               '/data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/'; 
% 
%   IDfile: 'IDfile'. Here "IDfile" is the ID number of each subject. It
%            should be is a txt file including the ID of each subject.  
% 
% thicknessdir = '/thickness/'; 
% 
% prefix = 'mci'; isnative = 1; fwhm = 30; rsl = 'rsl'
% 
% [M_left, M_right] = gretna_readthickness2(sourcedir, IDfile, thicknessdir,
% prefix, isnative, fwhm, rsl)
%
% If you choose the above input parameters, you will read the cortical
% thickness values from the following the files:
%
% /data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/01/thickness/
% mci_01_native_rms_rsl_tlink_30mm_left.txt  and ..._right.txt
%
% /data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/02/thickness/
% mci_02_native_rms_rsl_tlink_30mm_left.txt  and ..._right.txt ......
% 
% Yong HE, BIC, MNI, McGill 2006/09/12
% ************************************************************************

 
if nargin < 4
    error('No parameters');end

if nargin == 4 
    isnative = 1; fwhm = 30; rsl='rms'; end

if nargin == 5  
    fwhm = 30;  rsl ='rms'; end
if nargin == 6 
     rsl='rms';end

if nargin > 8 
    error('Error the number of input parameters (< 8).'); 
end

if ~strcmp(thicknessdir, '/thickness/') & ...
        ~strcmp(thicknessdir, '/thickness/')
    error('Error "thicknessdir" parameter');
end

if isnative == 1
    native = 'native';
elseif isnative ==0
    native = [];    
else
    error('Error "isnative" parameter (1 or 0)');
end

%switch fwhm
%    case {5,10,20,30,40}
%        fwhm = int2str(fwhm);
%    case {0}
%        fwhm = '';
%    otherwise
%        error('Unknown fwhm smooth kerna (0,5,10,20,30 or 40)l!');|
%end

if any(fwhm ==[0, 5, 20,30]) == 0
    error('Unknown fwhm smooth kerna (0,5,10,20,30 or 40)!');
    return;
end

subjects = textread(IDfile, '%s');
Nsub = size(subjects,1);

 %  Only include subjects for which files exist


rsl = 'rsl';
% side = 'left';
j = 1;
success_count = 0;
fail_count = 0;
for i = 1:Nsub
    % fprintf('.')	
    filename1 = strcat(strcat(strcat(strcat(prefix,'_'),subjects{i}),'_'),native);
    if fwhm == 0
        if strcmp(rsl, 'rsl') == 1
            filename2_left = '_rms_rsl_tlink_left.txt';
            filename2_right = '_rms_rsl_tlink_right.txt';
        else
            filename2_left = '_rms_tlink_left.txt';
            filename2_right = '_rms_tlink_right.txt';
        end
    else
        fwhmT = num2str(fwhm);
        if strcmp(rsl, 'rsl') == 1
            filename2_left = strcat(strcat('_rms_rsl_tlink_',fwhmT),'mm_left.txt');
            filename2_right = strcat(strcat('_rms_rsl_tlink_',fwhmT),'mm_right.txt');
        else
            filename2_left = strcat(strcat('_rms_tlink_',fwhmT),'mm_left.txt');
            filename2_right = strcat(strcat('_rms_tlink_',fwhmT),'mm_right.txt');
        end
    end
    filename3 = strcat(strcat(sourcedir,subjects{i}),thicknessdir);
    filename_left = strcat(filename3,strcat(filename1,filename2_left));
    filename_right = strcat(filename3,strcat(filename1,filename2_right));
   % try
        if (exist(filename_left, 'file') && exist(filename_right, 'file'))
            M_left(:,j) = load(filename_left);
            M_right(:,j) = load(filename_right);
            j = j + 1;
            success_count = success_count + 1;
            fprintf('.');
            else
   % catch
            fprintf('x')
            fail_count = fail_count + 1;
        end
end
M_left = M_left';
M_right = M_right';


% side = 'right';
% j = 0;
% for i = 1:Nsub
%     fprintf('.')	
%     filename1 = strcat(strcat(strcat(strcat(prefix,'_'),subjects{i}),'_'),native);
% %     filename2 =
% %     strcat(strcat('_rms_tlink_',fwhm),strcat(strcat('mm_',side),'.txt'));
%     if fwhm == 0
%         if strcmp(rsl,'rsl') == 1
% %            filename2 = strcat(strcat('_rms_rsl_tlink_',fwhm),strcat(strcat('mm_',side),'.txt'));
%              filename2 = strcat(strcat('_rms_rsl_tlink_',side),'.txt');
%         else
%             filename2 = strcat(strcat('_rms_tlink_',side),'.txt');
%         end
%     else        
%         if strcmp(rsl, 'rsl') == 1
%             filename2 = strcat(strcat('_rms_rsl_tlink_',fwhmT),strcat(strcat('mm_',side),'.txt'));
%         else
%             filename2 = strcat(strcat('_rms_tlink_',fwhmT),strcat(strcat('mm_',side),'.txt'));
%         end
%     end
%     filename3 = strcat(strcat(sourcedir,subjects{i}),thicknessdir);
%     filename = strcat(filename3,strcat(filename1,filename2));
%     try
%         M_right(:,j) = load(filename);
%         j = j + 1;
%         fprintf('.');
%     catch
%        fprintf('x') 
%     end
% end
% M_right = M_right';

fprintf('\nFinished.\n')
fprintf(strcat(num2str(success_count),' successes.\n'));
fprintf(strcat(num2str(fail_count),' failures.\n'));

