function [N, K, degree, sw] = gretna_sw_weighted(A, randtype, n)

% *************************************************************************
% This function is used to calculate the small-world properties (clustering
% and path length) and several other related measures of a graph G (i.e.
% network)  that is characterized by a weighted connectivity matrix A
% with nodes N and edges K.  Note that the small-world parameters were
% obtained based on the largest component of the graph G.
%
% function [N, K, degree, sw] = gretna_sw_weighted(A, randtype, n)
%
% input:
%           A:   
%                  A weighted connection/adjacency matrix (N*N, symmetric)
%           randtype:
%                   0: no random matrix was generated; only get the
%                      absolute values of Cp, Lp of the real netwok G; 
%                   1: The random matrix was generated using a weighted 
%                      version of Maslov's wiring program (Maslov and 
%                      Sneppen 2002). This resulting matrix has the same 
%                      number of nodes N, edges K and degree distribution 
%                      as the real network G. (default).
%                   2: The random matrix was generated using a general
%                   random algorithms. The resulting matrix has the same
%                   number of nodes N and edges K as the real network G.
%           n:    
%                   The number of generated random matrix (default: n =
%                   100)
% 
% output:
%           N:     
%                   The number of nodes in the graph G
%           K:     
%                   The number of edges in the graph G 
%    aveden:
%                   The average density (i.e. sparsity) in the graph G (the
%                   number of all its edges divided by the maximal possible
%                   number of edges (N^2-N)/2 in the graph G). 
%    degree.LeftNodeIndex:
%                   The index (serial number) of each node in the largest
%                   component of the graph G.
%    degree.aveWeight:
%                   The average weight in the graph G 
%       degree.Nweight:
%                   The weight of each node. (This measure
%                   is usually used to look at the connectivity weight
%                   distribution of the graph G).
%    sw.LargestCompSize:
%                   The size of the largestest component in the graph G
%    sw.CompN::
%                   The number of the components in the graph G (coming
%                   soon).
%            sw.Cp:
%                   The average clustering coefficient of the graph G (i.e.
%                   the average of the clustering coefficient over all
%                   nodes, where the clustering coefficient of a node is
%                   defined as the number of existing connections between
%                   the nodes neighbors divided by all their possible
%                   connections (Watts and Strogatz 1998).
%            sw.Lp:
%                   The global shortest path length of the graph G. That
%                   is, the smallest number of connections required to
%                   connect one node to another, averaged overall all pairs
%                   of nodes (Watts and Strogatz 1998).
%           sw.D:
%                   The shortest weighted path length matrix of the largest
%                   component in the graph G, where each element represents
%                   the shortest path weight between two nodes. 
%           sw.D0:
%                   The shortest path length matrix of the graph G, where
%                   each element represents the shortest binary path (all 
%                   paths have a weight of 1) length between two nodes.
%                   
%           sw.connectedness:
%                   The reachability number of any pairs of nodes.
%           sw.BIJ:
%                   The connection matrix in the largest component in the
%                   graph G. Note that, when there is only one compoent in
%                   the graph G, sw.BIJ = sw.AIJ.
%           sw.AIJ:
%                   The connection matrix of the graph G, i.e. the input
%                   matrix A (weighted). 
%    sw.meanCprand:
%                   The mean value of the clustering coefficient of n
%                   (default n = 100) random networks.
%     sw.stdCprand:
%                   The standard deviation of the clustering coefficient of
%                   n  (default n = 100) random networks.
%    sw.meanLprand:
%                   The mean value of the global shortest path length of n
%                    (default n = 100) random networks.
%     sw.stdLprand:
%                   The standard deviation of the global shortest path
%                   length of n  (default n = 100) random networks.
%         sw.Gamma:
%                   The relative clustering coefficient (Gamma =
%                   Cp/meanCprand)
%        sw.Lambda:
%                   The relative shortest path length (Lambda =
%                   Lp/meanLprand)
%         sw.Sigma:
%                   A scalar small-world parameter (Sigma = Gamma/Lambda)
% 
% For example: 
% 
% load ICBM_thickness_matrix.txt; 
% 
% [N, K, degree, sw] = gretna_sw(ICBM_thickness_matrix)
% 
% It is noted that a small-world network should met the following criteria:
% Gamma >>1, Lambda ~1, Sigma >>1 (Watts and Strogatz 1998; Humphries et
% al., 2005; He et al., 2006)
%
%
% References:
% 1. Watts DJ, Strogatz SH (1998) Collective dynamics of 'small-world'
% networks. Nature 393:440-442. 
% 
% 2. Maslov S, Sneppen K (2002) Specificity and stability in topology of
% protein networks. Science 296:910-913. 
% 
% 3. Humphries MD, Gurney K, Prescott TJ (2005) The brainstem reticular
% formation is a small world, not scale-free, network. Proc R Soc Lond B 
% Biol Sci, 273:503-511. 
% 
% 4.Newman MEJ. The structure and function of complex networks. Siam
% Rev 2003;45(2):167C256.
% 
% 5. He Y, Chen ZJ, Evans AC (2007) Small-word anatomical networks in the
% human brain revealed by cortical thickness from MRI. Cerebral Cortex, in
% press. 
%
% Zhang John Chen, Yong HE, BIC, MNI, McGill 2008/01/05
% *************************************************************************

 

if nargin < 1
    error('No parameters');end

if nargin == 1
    randtype =1; n = 100;end

if nargin == 2
    n = 100;end

if nargin > 3
    error('The number of parameters:<3'); end

A = A - diag(diag(A));
A = abs(A);
N = size(A,1);
K = sum(sum(A))/2;

% Measuring the properites of a real network G
% fprintf ('calculate the properties of the real networks\n')
[B, sw.LargestCompSize, sw.CompN, degree.LeftNodeIndex, degree.DelNodeIndex] = gretna_maxclique(A);
[degree.aveWeight, sw.N, sw.K, degree.Nweight] = gretna_degree(B);
% [aveden, N2, K1] = gretna_density(B);
[sw.Cp, sw.C] = gretna_clustcoeff_weighted(B);
[sw.Lp, sw.L] = gretna_shpathlength_weighted(B);
sw.BIJ = B;
sw.AIJ = A;
%sw.connectedness = length(find(sw.D0>0 & sw.D0~=inf))/2;
      
% fprintf ('-------Finished!-------\n');
if randtype~=0
    % measuring the properties of n random networks
    %     fprintf ('calculate the properties of the random networks (default: n = 100)\n')
    for i = 1:n
       % fprintf('.');
        if randtype == 1
            [randCIJ] =  sym_generate_srand(B);
        else
            [randCIJ] =  gretna_generate_random(sw.LargestCompSize,K);
        end
        [randCp(i), randC] = gretna_clustcoeff_weighted(randCIJ);
        [randLp(i), randD] = gretna_shpathlength_weighted(randCIJ);
    end
    sw.meanCprand = mean(randCp); sw.stdCprand = std(randCp);
    sw.meanLprand = mean(randLp); sw.stdLprand = std(randLp);
    sw.Gamma  = sw.Cp/sw.meanCprand;
    sw.Lambda  = sw.Lp/sw.meanLprand;
    sw.Sigma = sw.Gamma/sw.Lambda;
 
    %fprintf('\n');
    %if n < 20
      % fprintf('\nRecommend: Better generate at least 20 random networks.\n');
    %end
end %end if randtype

   
return


function [BIJ, LargestCompSize, CompN, leftNo, delNo] = gretna_maxclique(A_w);
% finding the largest component of the graph G (i.e. the binary matirx, A)
% BIJ: the matrix of the largest component of the graph G.
% LargestCompSize: the size of the largest component of the graph G
% CompN: the number of components of the graph G (coming soon).
% leftNo: the index (serial number) of each node in the largest component
% of the graph G.
% delNo, the index (serial number) of nodes that are not located in the
% largest component of the graph G.
%D0: The shortest path length matrix of the graph G, where each element
%represents the shortest binary path length between two nodes.

A = A_w;
A(find(A~=0)) = 1;
N = length(A);
K = length(find(A~=0));

[lambda, D] = gretna_shortestpathlength_binary(A);
    if max(max(D))~=inf
        BIJ = A_w;
        delNo = [];
        leftNo = 1:N;
        CompN = 1; LargestCompSize = N;
    else
        for i = 1:N
        maxpath(i) = length(find(D(i,:) ==inf));
        end
        delNo = find(maxpath ~=  min(maxpath));
        leftNo = find(maxpath == min(maxpath));
        CompN = [];
        LargestCompSize = length(leftNo);
        BIJ = A_w;
        BIJ(delNo,:)=[];
        BIJ(:,delNo)=[];      
    end  
return


function [aveWeight, N, K, ki] = gretna_degree(A);
% aveWeight: the average weight of the graph G
% N: number of node
% ki: weight of each node
% K: total weight of graph (one edge->one weight)
N = size(A,1);
ki = sum(A);
K = sum(ki)/2;
aveWeight = 2*K/N;
return


%function [aveden] = gretna_density(A)
% aveden: the average density of the graph G
%N = size(A,1);
%K = length(nonzeros(A))/2;
%aveden = K/((N^2-N)/2);
%return



function [avercc, C] = gretna_clustcoeff_weighted (W)
%avercc: the average clustering coefficient of the graph G. 
% cci: the clustering coefficiency of each node of the graph G.

%Onnela's weighted clustering coefficient for undirected graphs
%
%=========================================================================

K=sum(W~=0,2);            	
cyc3=diag((W.^(1/3))^3);           
K(cyc3==0)=inf;             %if no 3-cycles exist, make C=0 (via K=inf)
K(K==1)=inf;
C=cyc3./(K.*(K-1));         %clustering coefficient

avercc = mean(C);

return

function  [aveL, L] = gretna_shpathlength_weighted(A);

N = length (A);
L = all_shortest_paths(sparse(A));

L = triu(L,1);
aveL = sum(sum(L(L~=inf)))/(N*(N-1)/2);
L = L + L';

return

function  [lambda, D] = gretna_shortestpathlength_binary(A);
% lambda: the average shortest path length of the graph G
% D: the shortest path length matrix of the graph G. Each element indicates
% the shortest path length of two nodes

A = A + diag(diag(A));
A(find(A==0)) = inf;
N = length(A);
D = A - diag(diag(A));

% Floyd-Warshall algorithm
for i = 1:N
    k = setdiff((1:N),i);
    D(k,k) = min(D(k,k),repmat(D(k,i),1,N-1) + repmat(D(i,k),N-1,1));
end

D = triu(D,1);
lambda = sum(sum(D(D~=inf)))/(N*(N-1)/2);
D = D + D';
return





function [A] = gretna_generate_random(N,K);
% Network randomization method 1:
% The function is used to generate a random matrix with N nodes and K edges
% as the real network G.
B = ones(N,N);
B = B - diag(diag(B));
randp = randperm(N*(N-1)/2);
[rows,cols,vals] = find(tril(B));
A = sparse(rows(randp(1:K)), cols(randp(1:K)),ones(K,1),N,N);
A = full(A) + full(A)';
return


function [R] = sym_generate_srand(R);

%Undirected strongly-connected graph --> randomised undirected graph
%Each edge rewired ~ITER times; in/out degree and connectedness preserved
%Rewiring algorithm from Maslov and Sneppen (2002) Science 296:910
%
% Mika Rubinov, 2007
% =========================================================================


[i j]=find(tril(R));
K=length(i);
%ITER=K*ITER;
ITER = 2 * K;

for iter=1:ITER
    while 1                                     %while not rewired
        rewire=1;
        while 1
            e1=ceil(K*rand);
            e2=ceil(K*rand);
            while (e2==e1),
                e2=ceil(K*rand);
            end
            a=i(e1); b=j(e1);
            c=i(e2); d=j(e2);

            if all(a~=[c d]) && all(b~=[c d]);
                break           %all four vertices must be different
            end
        end

        %rewiring condition
        if ~(R(a,d) || R(c,b))
            %connectedness condition
            if ~(R(a,c) || R(b,d))
                P=R([a d],:);
                P(1,b)=0; P(2,c)=0;
                PN=P;
                PN(:,d)=1; PN(:,a)=1; 
                
                while 1
                    P(1,:)=any(R(P(1,:)~=0,:),1);
                    P(2,:)=any(R(P(2,:)~=0,:),1);
                    P=P.*(~PN);
                    if ~all(any(P,2))
                        rewire=0;
                        break
                    elseif any(any(P(:,[b c])))
                        break
                    end
                    PN=PN+P;
                end
            end %connectedness testing

            if rewire               %reassign edges
                R(a,d)=R(a,b); R(a,b)=0;
                R(d,a)=R(b,a); R(b,a)=0;
                R(c,b)=R(c,d); R(c,d)=0;
                R(b,c)=R(d,c); R(d,c)=0;

                j(e1) = d;          %reassign edge indices
                j(e2) = b;
                break;
            end %edge reassignment
        end %rewiring condition
    end %while not rewired
end %iterations

return