function [bmatrix,r] = mona_R2b_test (Rmatrix, type, thr)

% *************************************************************************
% [bmatrix,r] = mona_R2b(Rmatrix, type, thr)
%
% This function is used to threshold a correlation matrix into a binary
% matrix. Here, the thresholding can be selected in 3 different ways:
%
% 1) correlation threshold. For example, when selecting a correlation
% threshold of 0.3, the elements whose absolute values are larger than 0.3
% in the correlation matrix "Rmatrix" would be substitute for 1, otherwise
% 0.
% 
%           [bmatrix,r] = mona_R2b(Rmatrix, 'r', 0.3)
%
% 2) sparsity threshold. For example, when selecting a sparsity threshold
% of 0.1, the resulting binary matrix would have a sparsity of 0.1. Here
% the sparisty was defined as the number of edges divided by the maximum
% possible number of edges in a graph. It is noted that the elements whose
% values are "1" in the resulting binary matrix correspond to the top
% (0.1*N(N-1)/2) higher correlation values in the correlation matrix
% "Rmatrix". 
% 
%           [bmatrix,r] = mona_R2b(Rmatrix, 's', 0.1)
%
% 3) edge threshold. For example, when selecting a edge threshold of 200,
% the resulting binary matrix would have 200 edges. It is noted that the
% edges in the resulting binary matrix correspond to the top (0.1*N(N-1)/2)
% higher correlation values in the correlation matrix "Rmatrix". 
% 
%           [bmatrix,r] = mona_R2b(Rmatrix, 'k', 200)
%
% input:
%       Rmatrix:
%                   Rmatrix: symmetric correlation matrix.
%       type: 
%                   'r': correlation threshold; thr: the correlation value;
%                   's': sparsity threshold; thr: the sparsity value; 
%                   'k': edge threshold; thr: the number of edges;
% 
% output:
%       bmatrix: 
%                   The resulting binary matrix after thresholding the
%                   correlation matrix, "Rmatrix".
%                 r: 
%                   The corresponding correlation value at a sparsity
%                   or edge threshold. 
%
%
% Yong HE, BIC,MNI, McGill 2006/09/12
% *************************************************************************

N_region = length(Rmatrix);
Rmatrix = abs(Rmatrix);
%Rmatrix = Rmatrix - diag(diag(Rmatrix)); % removing the self-correlation
L_sw = length (thr);
bmatrix = zeros(L_sw, N_region*N_region);

if type == 's', % sparsity threshold
  %  if thr>1 | thr <=0
  %      error('0<thr <=1');
   % end
    sparsity = thr;
    K = round(sparsity*(N_region)*(N_region)/2);
end

%if type == 'k', %  edge threshold, i.e. the number of edges
  %  if thr <1 | thr >= (N_region)*(N_region-1)/2,
  %      error(' 1<=thr<N*(N-1)/2');
  %  end
%    K = thr;
%end

R_matrixretmp1 = reshape(Rmatrix, 1, N_region*N_region);
R_matrixretmp2 = repmat (R_matrixretmp1, L_sw, 1);
if type == 's' | type == 'k',
    [R_matrixrer1 R_matrixreindex1] = sort(R_matrixretmp2,2);
    for i = 1 : L_sw
        R_matrixtemp0 = R_matrixrer1(1,:);
        R_matrixindex_temp = R_matrixreindex1(1,:);
        R_matrixtemp0 (R_matrixindex_temp(1:(end-2*K(i)))) = 0;
        r(i) = R_matrixtemp0(R_matrixindex_temp(end-2*K(i)+1));
        R_matrixtemp0(R_matrixindex_temp((end-(2*K(i)-1)):end)) = 1;
%    bmatrix(i,:) = reshape(R_matrixretmp1,N_region,N_region);
        bmatrix(i,:) = R_matrixtemp0;
    end
end

if type == 'r', % correlation threshold
    for i = 1 : L_sw
         temp = R_matrixretmp1;
         temp (find(R_matrixretmp1>=thr(i))) = 1;
         temp (find(R_matrixretmp1<thr(i))) = 0;
         bmatrix(i,:) = temp;
    end
    r = thr;
end