function [Sparsity, Eff] = mona_effcost_sparsity (Rmatrix, s1, s2, deltas)

% *************************************************************************
% The function is used to calculate the global efficiency, wiring cost, and
% efficiency-cost (differences in global efficiency and cost) of
% small-world networks over a range of sparsity threshold (<0s1<s2<1)
% 
% function [Sparsity, Eff] = mona_effcost_sparsity (Rmatrix, s1, s2,
% deltas) 
% 
% input:
%           Rmatrix:
%                   correlation matrix (symmetric);
%           s1,s2,deltas:
%                   The sparsity threshold of the graph. 0<s1<=s2<1. deltas 
%                   = 0.1(default); 
% 
% output:
%          Sparsity:
%                   The sparsity of a network was defined as the number of
%                   connections divided by the maximum possible number of
%                   connections in the network. Here, the sparsity of a
%                   network is equal to its wiring cost.
%           Eff.cost:
%                   The cost of the networks at a large range of sparsity. 
%           Eff.gE:
%                   The global efficiency of the networks at a large range
%                   of sparisity
%           Eff.locE:
%                   The local efficiency of the networks at a large range
%                   of sparisity
%           Eff.gE_cost:
%                   The difference in global efficiency and cost of the
%                   networks at a large range of sparsity.
% 
% Example: 
%    Ploting the global efficiency, cost and efficiency-cost (difference
%    between efficiency and cost) of small-world over a range of sparsity
%    threshold of 0.01~0.80 at an interval of 0.01; 
%    
%   [sparsity eff] = mona_effcost_sparsity (R, 0.01, 0.80, 0.01)  
%    
%   plot(sparsity,eff.gE_cost,'-.c', sparsity,eff.gE,'-c',
%   sparsity,eff.cost,'-k')  
% 
% Yong HE, BIC,MNI, McGill 2007/05/01
% *************************************************************************


if nargin < 1
    error('No parameters');end

if nargin == 1
    N1 = size(Rmatrix);
    s1 = 0.01; s2 = 0.50; deltas = 0.02;end

if nargin > 5
    error('The number of input parameters should be less than 5.'); end


Rmatrix = abs(Rmatrix);
Rmatrix = Rmatrix - diag(diag(Rmatrix));

Sparsity = s1:deltas:s2
ns =length(Sparsity);

for m = 1:ns
    fprintf('\nm = %d\n',m);

    % global and local efficiency in the group1
    [corrmatrix1, r] = mona_R2b(Rmatrix, 's', Sparsity(m));
    [seff]  = mona_efficiency(corrmatrix1);
    Eff.gE(m) = seff.gE;
    Eff.loc(m) = seff.locE;
end

Eff.cost = Sparsity;
Eff.gE_cost = Eff.gE-Eff.cost;


