function [eff] = mona_efficiency (A)

% *************************************************************************
% This function is used to calculate the global and local efficiency of a
% graph (i.e. network) G (Latora and Marchiori, 2001) that are
% characterized by a binarized connectivity matrix A with nodes N and edges
% K. 
%
% function [eff] = mona_efficiency(A)
%
% input:
%           A: 
%               a binary matrix (symmetric).
%
% output:
%       eff.gE:
%               global efficiency of a graph G (corresponding to global
%               average path length). 
%       eff.locE:
%               local efficiency of a graph G (corresponding to average
%               clustering coefficient). 
%       eff.regionallocalE:
%               regionally local efficiency of each node in a graph G (like
%               clustering coefficient of each node).
%       eff. regionalglobalE:
%               regionally nodal efficiency of each node in a graph G.
%               (This measure can be used to identify hub nodes of the
%               graph G. A high value indicates a high ability to manage
%               information flow of the network).
% 
% Reference: 
% 1. Latora V, Marchiori M (2001) Efficient behavior of small-world
% networks. Phys Rev Lett 87: 198701. 
%
% Yong HE, BIC, MNI, McGill 2007/05/01
% *************************************************************************


warning off

N = length(A);
[D0] = mona_shpathlength(A);
D0 = D0 + diag(diag(1./zeros(N,N)));
eff1 = 1./D0;

% global efficiency of a graph
eff.gE = sum(sum(eff1))/(N*(N-1));

% global efficiency of each node;
eff.regionalglobalE = sum(eff1,2)/(N-1);

for i=1:N
    NV =  find(A(i,:));
    if length(NV)==0 | length(NV)==1
        regionallocalE(i,1) = 0;
    else
        B = A(NV,NV);
        [D1] = mona_shpathlength(B);
        D1 = D1 + diag(diag(1./zeros(size(D1))));
        localeff1 = 1./D1;

        % local efficiency of each node
        regionallocalE(i,1)  = sum(sum(localeff1))/(length(NV)*(length(NV)-1)) ;
    end
end

localE = sum(regionallocalE)/N;
eff.locE = localE;
eff.regionallocalE = regionallocalE;

return



function  [D] = mona_shpathlength(A);
A = A + diag(diag(A));
A(find(A==0)) = inf;
N = length(A);
D = A - diag(diag(A));

% Floyd-Warshall algorithm
for i = 1:N
    k = setdiff((1:N),i);
    D(k,k) = min(D(k,k),repmat(D(k,i),1,N-1) + repmat(D(i,k),N-1,1));
end
D = triu(D,1) + tril(D,-1);
return
