function [vertex_animalmidsurf_left, vertex_animalmidsurf_right] = ...
                    mona_readanimalmidsurf2(sourcedir, IDfile, surfacedir, prefix)

% *************************************************************************
% This function is used to read animal mid cortical surface (ANIMAL +
% INSECT) typically obtained from the CIVET pipeline procedure (Claude's
% version).
% 
% function [vertex_animalmidsurf_matrix_left,vertex_animalmidsurf_matrix_right] =  
%                   mona_readanimalmidsurf2 (sourcedir, IDfile, surfacedir, prefix)  
%
% input:
%        sourcedir:  
%                       the directory of each subject.
%                       e.g./data/aces/aces1/yonghe/eld_mci/pipe1004/outpu
%                       tmci/;
%               subID:
%                       a txt file including the ID of each subject. It
%                       should be the name of subdirectory under the input
%                       parameter "sourcedir". If you open this txt file.
%                       It should be like this: 01 02 03 04... 
%                       or like this:
%                           01 
%                           02 
%                           03 
%                           04 
%                           ...
%           surfacedir: 
%                       the directory saving the anaimal_midsurface for
%                       each subject. It should be "/segment/" ;
%               prefix:      
%                       the prefix name of animal midsurface file. e.g. in
%                       the filename
%                       "mci_01_animal_surface_labels_left.txt", 
%                       prefix = 'mci'; 
%
% output:
%           vertex_animalmidsurf_left: 
%                       an s*n matrix with animal midsurface values for
%                       each node for each subject (left hemisphere). n:
%                       the number of nodes (e.g. 40962); s: the number of
%                       subjects.
%           vertex_animalmidsurf_right: 
%                       an s*n matrix with animal midsurface values for
%                       each node for each subject (right hemisphere). n:
%                       the number of nodes (e.g. 40962); s: the number of
%                       subjects. 
% 
% 
% For example: 
% 
% sourcedir =
%           '/data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/'; 
% 
%   IDfile: 'IDfile'. Here "IDfile" is the ID number of each subject. It
%            should be is a txt file including the ID of each subject. 
% 
%   surfacedir = '/segment/';  prefix = 'mci';    
% 
% [vertex_animalmidsurf_matrix_left,vertex_animalmidsurf_matrix_right] =
% ... mona_readanimalmidsurf2(sourcedir, IDfile, surfacedir, prefix)
%
% /data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/01/segment/
% mci_01_animal_surface_labels_left.txt and ..._right.txt
%
% /data/aces/aces1/yonghe/eld_mci/pipe1004/outputmci/02/segment/
% mci_02_animal_surface_labels_left.txt and ..._right.txt
%
% Yong HE, BIC, MNI, McGill 2006/09/12
% ************************************************************************


if nargin ~= 4
    error('No parameters');end

if ~strcmp(surfacedir, '/segment/')
    error('Error "surfacedir" parameter');
end

subID = textread(IDfile, '%s');
Nsub = size(subID,1);

side = 'left';
for i = 1:Nsub
    fprintf('.')
    filename1 = strcat(strcat(strcat(prefix,'_'),subID{i}),'_animal_surface_labels_',strcat(side,'.txt'));
    filename2 = strcat(strcat(sourcedir,subID{i}),surfacedir);
    filename = strcat(filename2,filename1);
    vertex_animalmidsurf_left(i,:) = load(filename)';
end

 
side = 'right';
for i = 1:Nsub
    fprintf('.')
    filename1 = strcat(strcat(strcat(prefix,'_'),subID{i}),'_animal_surface_labels_',strcat(side,'.txt'));
    filename2 = strcat(strcat(sourcedir,subID{i}),surfacedir);
    filename = strcat(filename2,filename1);
    vertex_animalmidsurf_right(i,:) = load(filename)';
end
  
fprintf('Finished read anaimial surface.\n')
