function [N, K, degree, sw] = mona_sw_harmonic(A, randtype, n)

% *************************************************************************
% This function is used to calculate the small-world properties (clustering
% and "harmonic mean" path length) and several other related measures of a
% graph (i.e. network) G that is characterized by a binarized connectivity
% matrix A with nodes N and edges K. Note that the small-world properties
% were calcualted based on the WHOLE graph.
%     
%
% function [N, K, degree, sw] = mona_sw_harmonic(A, randtype, n)
%
% input:
%           A:   
%                  A binarized connection/adjacency matrix (N*N, symmetric)
%        randtype:
%                   0: no random matrix was generated; only get the
%                   absolute values of Cp, Lp of the real netwok G; 1: The
%                   random matrix was generated using Maslov's wiring
%                   program (Maslov and Sneppen 2002). This resulting
%                   matrix has the same number of nodes N, edges K and
%                   degree distribution as the real network G. (default).
%                   2: The random matrix was generated using a general
%                   random algorithms. The resulting matrix has the same
%                   number of nodes N and edges K as the real network G.
%           n:    
%                   The number of generated random matrix (default: n =
%                   100)
% 
% output:
%           N:     
%                   The number of nodes in the graph G
%           K:     
%                   The number of edges in the graph G 
%    aveden:
%                   The average density (i.e. sparsity) in the graph G (the
%                   number of all its edges divided by the maximal possible
%                   number of edges (N^2-N)/2 in the graph G). 
%    degree.avedeg:
%                   The average connectivity (degree) in the graph G (i.e.
%                   the average of the degree over all nodes, where the
%                   degree of a node corresponds to the number of
%                   connections to that node).
%       degree.deg:
%                   The number of connections for each node. (This measure
%                   is usually used to look at the connectivity degee
%                   distribution of the graph G).
%            sw.Cp:
%                   The average clustering coefficient of the graph G (i.e.
%                   the average of the clustering coefficient over all
%                   nodes, where the clustering coefficient of a node is
%                   defined as the number of existing connections between
%                   the nodes neighbors divided by all their possible
%                   connections (Watts and Strogatz 1998). If a node i has
%                   only an edge or no edges, so C(i) = 0;
%            sw.Lp:
%                   The global "harmonic" shortest path length of the graph
%                   G. 1/Lp = sum(sum(1/dij))/(N*(N-1). This modified path
%                   length is slightly different from the traditional path
%                   length, i.e. Lp = sum(sum(dij))/(N*(N-1)). This
%                   modififed definition has advantages in calculating the
%                   properties of a network with multiple components
%                   (Newman 2003).   
%           sw.D0:
%                   The shortest path length matrix of the graph G, where
%                   each element represents the shortest path length
%                   between two nodes.
%           sw.AIJ:
%                   The connection matrix of the graph G, i.e. the input
%                   matrix A (binary). 
%    sw.meanCprand:
%                   The mean value of the clustering coefficient of n
%                   (default n = 100) random networks.
%     sw.stdCprand:
%                   The standard deviation of the clustering coefficient of
%                   n  (default n = 100) random networks.
%    sw.meanLprand:
%                   The mean value of the global shortest path length of n
%                    (default n = 100) random networks.
%     sw.stdLprand:
%                   The standard deviation of the global shortest path
%                   length of n  (default n = 100) random networks.
%         sw.Gamma:
%                   The relative clustering coefficient (Gamma =
%                   Cp/meanCprand)
%        sw.Lambda:
%                   The relative shortest path length (Lambda =
%                   Lp/meanLprand)
%         sw.Sigma:
%                   A scalar small-world parameter (Sigma = Gamma/Lambda)
% 
% For example: 
% 
% load ICBM_thickness_matrix.txt; 
% 
% 1. [N, K, degree, sw] = mona_sw_harmonic(ICBM_thickness_matrix)
% 
% 2. [N, K, degree, sw] = mona_sw_harmonic(ICBM_thickness_matrix,
%    1, 1000) 
% 3. [N, K, degree, sw] = mona_sw_harmonic(ICBM_thickness_matrix,
%    2, 1000) 
% 
% It is noted that a small-world network should met the following criteria:
% Gamma >>1, Lambda ~1, Sigma >>1 (Watts and Strogatz 1998; Humphries et
% al., 2005; He et al., 2006)
%
%
% References:
% 1. Watts DJ, Strogatz SH (1998) Collective dynamics of 'small-world'
% networks. Nature 393:440-442. 
% 
% 2. Maslov S, Sneppen K (2002) Specificity and stability in topology of
% protein networks. Science 296:910-913. 
% 
% 3. Humphries MD, Gurney K, Prescott TJ (2005) The brainstem reticular
% formation is a small world, not scale-free, network. Proc R Soc Lond B 
% Biol Sci, 273:503-511. 
% 
% 4.Newman MEJ. The structure and function of complex networks. Siam
% Rev 2003;45(2):167C256.
% 
% 5. He Y, Chen ZJ, Evans AC (2007) Small-word anatomical networks in the
% human brain revealed by cortical thickness from MRI. Cerebral Cortex, in
% press. 
%
% Yong HE, BIC, MNI, McGill 2007/05/01
% *************************************************************************

 

if nargin < 1
    error('No parameters');end

if nargin == 1
    randtype =1; n = 100;end

if nargin == 2
    n = 100;end

if nargin > 3
    error('The number of parameters:<3.'); end

% Measuring the properites of a real network G
% fprintf ('calculate the properties of the real networks\n')
warning off;
A = A - diag(diag(A));
N = length(A);
[D0] = mona_shortestpathlength(A);
D0 = D0 + diag(diag(1./zeros(N,N)));
sw.Lp = 1/(sum(sum(1./D0))/(N*(N-1)));
[degree.avedeg, K, degree.deg] = mona_degree(A);
% [aveden] = mona_density(A);
[sw.Cp, sw.Ci] = mona_clustcoeff(A);
sw.AIJ = A;

if randtype~=0
    % measuring the properties of n random networks
    for i = 1:n
        fprintf('.');
        if randtype == 1
            [randCIJ] =  sym_generate_srand(A);
        else
            [randCIJ] =  mona_generate_random(N,K);
        end
        [randCp(i), randC] = mona_clustcoeff(randCIJ);
        [randD0] = mona_shortestpathlength(randCIJ);
        randD0 = randD0 + diag(diag(1./zeros(N,N)));
        randLp(i) = 1/(sum(sum(1./randD0))/(N*(N-1)));        
    end
    sw.Gamma  = sw.Cp/mean(randCp);
    sw.Lambda  = sw.Lp/mean(randLp);
    sw.Sigma = sw.Gamma/sw.Lambda;
end
return
 


function [avedeg, K, ki] = mona_degree(A);
%avedeg: the average degree of the graph G and degree of each node
N = size(A,1);
ki = sum(A);
K = sum(ki)/2;
avedeg = 2*K/N;
return


function [aveden] = mona_density(A)
% aveden: the average density of the graph G
N = size(A,1);
K = length(nonzeros(A))/2;
aveden = K/((N^2-N)/2);
return



function [avercc, cci] = mona_clustcoeff (A)
%avercc: the average clustering coefficient of the graph G. 
% cci: the clustering coefficiency of each node of the graph G.
N = size(A,1);
cci = [];

for i = 1:N
    NV = find(A(i,:));
    if length(NV) == 1 | length(NV) == 0
        cc = 0;
    else
        cc = ((sum(sum(A(NV,NV))))/2)/((length(NV)^2-length(NV))/2);
    end
    cci = [cci cc];
end
avercc = mean(cci);
return



function  [D] = mona_shortestpathlength(A);
% lambda: the average shortest path length of the graph G
% D: the shortest path length matrix of the graph G. Each element indicates
% the shortest path length of two nodes
A = A + diag(diag(A));
A(find(A==0)) = inf;
N = length(A);
D = A - diag(diag(A));

% Floyd-Warshall algorithm
for i = 1:N
    k = setdiff((1:N),i);
    D(k,k) = min(D(k,k),repmat(D(k,i),1,N-1) + repmat(D(i,k),N-1,1));
end
D = triu(D,1) + tril(D,-1);
return


function [A] = mona_generate_random(N,K);
% Network randomization method 1:
% The function is used to generate a random matrix with N nodes and K edges
% as the real network G.
B = ones(N,N);
B = B - diag(diag(B));
randp = randperm(N*(N-1)/2);
[rows,cols,vals] = find(tril(B));
A = sparse(rows(randp(1:K)), cols(randp(1:K)),ones(K,1),N,N);
A = full(A) + full(A)';
return


function [srand] = sym_generate_srand(A);
% Network randomization method 2:
% This function is used to generate a random network with the same N, K and
% degree distribution as a  real network using Maslovs wiring algorithm
% (Maslov et al. 2002). This function is slightly revised according to
% Maslov's wiring program  (http://www.cmth.bnl.gov/~maslov/).
% Yong HE,
srand = A;
srand = srand - diag(diag(srand));
nrew = 0;

[i1,j1]=find(srand);
aux=find(i1>j1);
i1=i1(aux);
j1=j1(aux);
Ne=length(i1);

ntry=2*Ne;

for i=1:ntry
    e1=1+floor(Ne*rand);
    e2=1+floor(Ne*rand);
    v1=i1(e1);
    v2=j1(e1);
    v3=i1(e2);
    v4=j1(e2);
    if srand(v1,v2)<1;
        v1
        v2
        srand(v1,v2)
        pause;
    end;
    if srand(v3,v4)<1;
        v3
        v4
        srand(v3,v4)
        pause;
    end;

    if (v1~=v3)&(v1~=v4)&(v2~=v4)&(v2~=v3);
        if rand>0.5;
            if (srand(v1,v3)==0)&(srand(v2,v4)==0);

                % the following line prevents appearance of isolated clusters of size 2
                %           if (k1(v1).*k1(v3)>1)&(k1(v2).*k1(v4)>1);

                srand(v1,v2)=0;
                srand(v3,v4)=0;
                srand(v2,v1)=0;
                srand(v4,v3)=0;

                srand(v1,v3)=1;
                srand(v2,v4)=1;
                srand(v3,v1)=1;
                srand(v4,v2)=1;

                nrew=nrew+1;

                i1(e1)=v1;
                j1(e1)=v3;
                i1(e2)=v2;
                j1(e2)=v4;

                % the following line prevents appearance of isolated clusters of size 2
                %            end;

            end;
        else
            v5=v3;
            v3=v4;
            v4=v5;
            clear v5;

            if (srand(v1,v3)==0)&(srand(v2,v4)==0);

                % the following line prevents appearance of isolated clusters of size 2
                %           if (k1(v1).*k1(v3)>1)&(k1(v2).*k1(v4)>1);

                srand(v1,v2)=0;
                srand(v4,v3)=0;
                srand(v2,v1)=0;
                srand(v3,v4)=0;

                srand(v1,v3)=1;
                srand(v2,v4)=1;
                srand(v3,v1)=1;
                srand(v4,v2)=1;

                nrew=nrew+1;

                i1(e1)=v1;
                j1(e1)=v3;
                i1(e2)=v2;
                j1(e2)=v4;

                % the following line prevents appearance of isolated clusters of size 2
                %           end;

            end;
        end;
    end;
end;

return




