function [Sparsity, PathLen, ClustCoff] =...
    mona_swdiff_sparsity (group1, group2, s1, s2, deltas, num_rand, brandnetwork)

% *************************************************************************
% The function is used to calculate the differences in the small-world
% parameters between the groups (e.g. normal vs. patient) over a large
% range of sparsity threshold. For example, when selecting a given sparisty
% threshold (e.g. 0.1) for the two groups, we can calculate the small-world
% properties (clustering and shortest path length) of each network and
% compare the between-group difference using random permutation methods. It
% should be noted that the edges of each thresholded network are those
% connections showing the top higher correlations (e.g. 0.1*(N*(N-1)/2), N
% is the number of brain regions) in each group. If there are significant
% differences in small-world properties, so we can conclude that the
% small-world differences might reflect the different brain organization
% between the groups since the two networks have the same number of nodes
% and sparsity (corresponding to the same number of edges).
%
% function [Sparsity, PathLen, ClustCoff, rerand] =
%               mona_swdiff_sparsity(group1, group2, s1, s2, deltas, num_rand) 
%
% input:
%           group1:
%                   cortical thickness matrix of group1 (s*N ). s: the
%                   number of subjects in group1; N: the number of brain
%                   regions (54 cortical regions in the jacob template).   
%           group2:
%                   cortical thickness matrix of group1 (s*N ). s: the
%                   number of subjects in group2; N: the number of brain
%                   regions. 
%           s1,s2,deltas:
%                   The sparsity threshold of the graph. 0<s1<=s2<1. deltas
%                   = 0.1(default); 
%           num_rand:
%                   The number of random permutations
%           brandnetwork:
%                   0: no random networks; only calucating the differneces
%                   in the Cp and Lp values between the two groups; 1:
%                   generating random networks to calucate the differences
%                   in the Gamma and Lambda values between the two groups
%                   while calcuating small-world attributes. 
% 
% output:
%           PathLen.Lp1, PathLen.Lp2:
%                   The global average shortest path length of the
%                   group1,group2 networks over a large range of sparsity
%                   threshold.
%          PathLen.lower95Lp12diff, PathLen.upper95Lp12diff:
%                  When performing 1000 random permutation test, the 95
%                  percentile points of the distribution of between-group
%                  differences in path length.
%           PathLen.lower95Lambda12diff, PathLen.upper95Lambda12diff:
%                  When performing 1000 random permutation test, the 95
%                  percentile points of the distribution of between-group
%                  differences in path length ratio (real/random).
%          ClustCoff.Cp1, ClustCoff.Cp2:
%                   The global average cluster coefficient of the group1,
%                   group2 network over a large range of sparsity
%                   threshold.
%         ClustCoff.lower95Cp12diff, ClustCoff.upper95Cp12diff:
%                  When performing 1000 random permutation test, the 95
%                  percentile points of the distribution of between-group
%                  differences in clustering coefficient.
%     ClustCoff.lower95Gamma12diff, ClustCoff.upperer95Gamma12diff:
%                  When performing 1000 random permutation test, the 95
%                  percentile points of the distribution of between-group
%                  differences in clustering coefficient ratio.
%     Sigma.lower95Sigma12diff, Sigma.upperer95Sigma12diff:
%                  When performing 1000 random permutation test, the 95
%                  percentile points of the distribution of between-group
%                  differences in small-worldness.
%           Sparsity:
%                  The sparsity of a network was defined as the number of
%                  connections divided by the maximum possible number of
%                  connections in the network. Here it should be the same
%                  for the two groups.  
%
% Examples: 
% 1. calculating the small-world differences between-group at an
%     sparsity threshold of 0.1; (1000 permutation test). 
% 
%     [Sparsity, PathLen, ClustCoff, rerand] = mona_swdiff_sparsity
%     (group1, group2, 0.1, 0.1, 0.2, 1000);  
% 
% 2. calculating the small-world differences between-group over a range of
%     sparsity threshold of 0.01~0.4 at an interval of 0.02; (1000
%     permutation test). 
%    
%     [Sparsity, PathLen, ClustCoff, rerand] =  mona_swdiff_sparsity
%     (group1, group2, 0.01, 0.4, 0.02, 1000); 
% 
% 
% Yong HE, BIC,MNI, McGill 2007/05/01
% *************************************************************************


if nargin < 2
    error('No parameters');end

if nargin == 2
    N1 = size(group1,2);
    s1 = 0.01; s2 = 0.40; deltas = 0.02; num_rand = 400;end

if nargin == 6
    brandnetwork = 0;end

if nargin > 7
    error('The number of parameters:<7'); end

N1 = size(group1,1);
N2 = size(group2,1);
N_tgroup = N1 + N2;
tgroup_Reg = [group1;group2];

R_matrix1 = corrcoef(group1);
R_matrix2 = corrcoef(group2);

N_region = size(R_matrix1,1);
R_matrix1 = abs(R_matrix1);
R_matrix2 = abs(R_matrix2);
R_matrix1 = R_matrix1 - diag(diag(R_matrix1));
R_matrix2 = R_matrix2 - diag(diag(R_matrix2));

% caluculate the Cp, Lp of nc, ad group at different average degree <K> and
% compare the between-group significance of Cp, Lp  by a randomized
% procedure

Sparsity = s1:deltas:s2


for m = 1:length(Sparsity)
    fprintf('\nm = %d\n',m);

    % Cp Lp in the group1
    [corrmatrix1, r] = mona_R2b(R_matrix1, 's', Sparsity(m));
    if brandnetwork == 0
        [NN, KK,  aveden, degree, sw]  = mona_sw_harmonic(corrmatrix1,0,0);
    else
        [NN, KK,  aveden, degree, sw]  = mona_sw_harmonic(corrmatrix1,1,50);
        PathLen.Lambda1(m) = sw.Lambda;
        ClustCoff.Gamma1(m) = sw.Gamma;
        Sigma.Sigma1 (m) = sw.Sigma;
    end
    PathLen.Lp1(m) = sw.Lp;  
    ClustCoff.Cp1(m) = sw.Cp;

    % Cp Lp in the group2
    [corrmatrix2, r] = mona_R2b(R_matrix2, 's', Sparsity(m));
    if brandnetwork == 0
        [NN, KK,  aveden, degree, sw]  = mona_sw_harmonic(corrmatrix2,0,0);
    else
        [NN, KK,  aveden, degree, sw]  = mona_sw_harmonic(corrmatrix2,1,50);
        PathLen.Lambda2(m) = sw.Lambda;
        ClustCoff.Gamma2(m) = sw.Gamma;
        Sigma.Sigma2 (m) = sw.Sigma;
    end
    PathLen.Lp2(m) = sw.Lp;    
    ClustCoff.Cp2(m) = sw.Cp;

    % a randomization procedure is used to detect the significance of the
    % difference of Cp Lp
    for i = 1: num_rand
        fprintf('-')
        rp = randperm(N_tgroup);
        rerand_Reg1 = tgroup_Reg(rp(1:N1),:);
        rerand_Reg2 = tgroup_Reg(rp ((N1+1):N_tgroup),:);
        rerand_corr_1 = corrcoef(rerand_Reg1);
        rerand_corr_2 = corrcoef(rerand_Reg2);
        rerand_corr_1 = abs(rerand_corr_1) ;
        rerand_corr_2 = abs(rerand_corr_2) ;

        rerand_corr_1 = rerand_corr_1 - diag(diag(rerand_corr_1));
        rerand_corr_2 = rerand_corr_2 - diag(diag(rerand_corr_2));

        [corrmatrix1, r] = mona_R2b(rerand_corr_1, 's', Sparsity(m));
        if brandnetwork == 0
            [NN, KK,  aveden, degree, sw]  = mona_sw_harmonic(corrmatrix1,0,0);
        else
            [NN, KK,  aveden, degree, sw]  = mona_sw_harmonic(corrmatrix1,1,50);
            rerand_Lambda1(i) = sw.Lambda;
            rerand_Gamma1(i) = sw.Gamma;
            rerand_Sigma1(i) = sw.Sigma;
        end
        rerand_Lp1(i) = sw.Lp; 
        rerand_Cp1(i) = sw.Cp;

        [corrmatrix2, r] = mona_R2b(rerand_corr_2, 's', Sparsity(m));
        if brandnetwork == 0
            [NN, KK,  aveden, degree, sw]  = mona_sw_harmonic(corrmatrix2,0,0);
        else
            [NN, KK,  aveden, degree, sw]  = mona_sw_harmonic(corrmatrix2,1,50);
            rerand_Lambda2(i) = sw.Lambda;
            rerand_Gamma2(i) = sw.Gamma;
            rerand_Sigma2(i) = sw.Sigma;
        end
        rerand_Lp2(i) = sw.Lp;
        rerand_Cp2(i) = sw.Cp;
    end

    rerand_Lp12_diff = rerand_Lp1-rerand_Lp2;
    rerand_Lp12_diff = rerand_Lp12_diff(find(isnan(rerand_Lp12_diff)==0));
    rerand_Cp12_diff = rerand_Cp1-rerand_Cp2;
    rerand_Cp12_diff = rerand_Cp12_diff(find(isnan(rerand_Cp12_diff)==0));

    deltaLp = PathLen.Lp1(m) - PathLen.Lp2(m);
    if deltaLp >=0
        PathLen.Lp12_diff_p(m) = length(find(rerand_Lp12_diff>deltaLp))/(length(rerand_Lp12_diff)+1);
    else
        PathLen.Lp12_diff_p(m) = length(find(rerand_Lp12_diff<deltaLp))/(length(rerand_Lp12_diff)+1);
    end
    [sortmaxminLp12diff sortmaxminLp12diffindex] = sort(rerand_Lp12_diff);
    PathLen.lower95Lp12diff (m)= sortmaxminLp12diff(ceil((num_rand+1)*0.05));
    PathLen.upper95Lp12diff (m) = sortmaxminLp12diff(end-ceil((num_rand+1)*0.05)-1);

    deltaCp = ClustCoff.Cp1(m) - ClustCoff.Cp2(m);
    if deltaCp >=0
        ClustCoff.Cp12_diff_p(m) = length(find(rerand_Cp12_diff>deltaCp))/(length(rerand_Cp12_diff)+1);
    else
        ClustCoff.Cp12_diff_p(m) = length(find(rerand_Cp12_diff<deltaCp))/(length(rerand_Cp12_diff)+1);
    end
    [sortmaxminCp12diff sortmaxminCp12diffindex] = sort(rerand_Cp12_diff);
    ClustCoff.lower95Cp12diff (m) = sortmaxminCp12diff(ceil((num_rand+1)*0.05));
    ClustCoff.upper95Cp12diff (m) = sortmaxminCp12diff(end-ceil((num_rand+1)*0.05)-1);
    
    
    
    
    if brandnetwork ==1
        rerand_Lambda12_diff = rerand_Lambda1-rerand_Lambda2;
%         rerand_Lambda12_diff =
%         rerand_Lambda12_diff(find(isnan(rerand_Lambda12_diff)==0));
        rerand_Gamma12_diff = rerand_Gamma1-rerand_Gamma2;
%         rerand_Gamma12_diff = rerand_Gamma12_diff(find(isnan(rerand_Gamma12_diff)==0));
        rerand_Sigma12_diff = rerand_Sigma1-rerand_Sigma2;
        
        deltaLambda = PathLen.Lambda1(m) - PathLen.Lambda2(m);
        if deltaLp >=0
            PathLen.Lambda12_diff_p(m) = length(find(rerand_Lambda12_diff>deltaLambda))/(length(rerand_Lambda12_diff)+1);
        else
            PathLen.Lambda12_diff_p(m) = length(find(rerand_Lambda12_diff<deltaLambda))/(length(rerand_Lambda12_diff)+1);
        end
        [sortmaxminLambda12diff sortmaxminLambda12diffdiffindex] = sort(rerand_Lambda12_diff);
        PathLen.lower95Lambda12diff  (m) =  sortmaxminLambda12diff(ceil((num_rand+1)*0.05));
        PathLen.upper95Lambda12diff  (m) =  sortmaxminLambda12diff(end-ceil((num_rand+1)*0.05)-1);

        deltaGamma = ClustCoff.Gamma1(m) - ClustCoff.Gamma2(m);
        if deltaGamma >=0
            ClustCoff.Gamma12_diff_p(m) = length(find(rerand_Gamma12_diff>deltaGamma))/(length(rerand_Gamma12_diff)+1);
        else
            ClustCoff.Gamma12_diff_p(m) = length(find(rerand_Gamma12_diff<deltaGamma))/(length(rerand_Gamma12_diff)+1);
        end
        [sortmaxminGamma12diff sortmaxminGamma12diffindex] = sort(rerand_Gamma12_diff);
        ClustCoff.lower95Gamma12diff   (m) = sortmaxminGamma12diff(ceil((num_rand+1)*0.05));
        ClustCoff.upper95Gamma12diff  (m) = sortmaxminGamma12diff(end-ceil((num_rand+1)*0.05)-1);


        deltaSigma = Sigma.Sigma1(m) - Sigma.Sigma2(m);
        if deltaSigma >=0
            Sigma.Sigma12_diff_p(m) = length(find(rerand_Sigma12_diff>deltaSigma))/(length(rerand_Sigma12_diff)+1);
        else
            Sigma.Sigma12_diff_p(m) = length(find(rerand_Sigma12_diff<deltaSigma))/(length(rerand_Sigma12_diff)+1);
        end
        [sortmaxminSigma12diff sortmaxminSigma12diffindex] = sort(rerand_Sigma12_diff);
        Sigma.lower95Sigma12diff   (m) = sortmaxminSigma12diff(ceil((num_rand+1)*0.05));
        Sigma.upper95Sigma12diff  (m) = sortmaxminSigma12diff(end-ceil((num_rand+1)*0.05)-1);
%         rerand.Sigma{m} = rerand_gE12_diff;
    end

end


% the area under the Cp,Lp, Gamma, Lambda and Sigma curves
AUC.aCp1 = sum(ClustCoff.Cp1)*(Sparisity(end)-Sparsity(1));
AUC.aCp2 = sum(ClustCoff.Cp2)*(Sparisity(end)-Sparsity(1));
AUC.aCpdiff = AUC.aCp1 - AUC.aCp2;
% 
% for i = 1:num_rand
%     rand_aCp1 = 
% 
% 
% 
%    PathLen.Lambda1(m) = sw.Lambda;
%         ClustCoff.Gamma1(m) = sw.Gamma;
%         Sigma.Sigma1 (m) = sw.Sigma;
%     end
%     PathLen.Lp1(m) = sw.Lp;  
%     ClustCoff.Cp1(m) = sw.Cp;